/*
 * (c) Urs Zeidler 2011
 */
package de.urszeidler.eclipse.callchain.components;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.BasicDiagnostic;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.WrappedException;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.Diagnostician;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.eef.runtime.api.notify.IPropertiesEditionEvent;
import org.eclipse.emf.eef.runtime.context.PropertiesEditingContext;
import org.eclipse.emf.eef.runtime.context.impl.EObjectPropertiesEditionContext;
import org.eclipse.emf.eef.runtime.context.impl.EReferencePropertiesEditionContext;
import org.eclipse.emf.eef.runtime.impl.components.SinglePartPropertiesEditingComponent;
import org.eclipse.emf.eef.runtime.impl.filters.EObjectFilter;
import org.eclipse.emf.eef.runtime.impl.notify.PropertiesEditionEvent;
import org.eclipse.emf.eef.runtime.impl.utils.EEFConverterUtil;
import org.eclipse.emf.eef.runtime.policies.PropertiesEditingPolicy;
import org.eclipse.emf.eef.runtime.policies.impl.CreateEditingPolicy;
import org.eclipse.emf.eef.runtime.providers.PropertiesEditingProvider;
import org.eclipse.emf.eef.runtime.ui.widgets.ButtonsModeEnum;
import org.eclipse.emf.eef.runtime.ui.widgets.eobjflatcombo.EObjectFlatComboSettings;
import org.eclipse.emf.eef.runtime.ui.widgets.referencestable.ReferencesTableSettings;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;

import de.urszeidler.eclipse.callchain.AtlLibary;
import de.urszeidler.eclipse.callchain.AtlRes;
import de.urszeidler.eclipse.callchain.Callable;
import de.urszeidler.eclipse.callchain.CallchainPackage;
import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.Property;
import de.urszeidler.eclipse.callchain.parts.CallchainViewsRepository;
import de.urszeidler.eclipse.callchain.parts.CallsPropertiesEditionPart;


/**
 * @author Urs Zeidler
 * @generated
 */
public class CallsPropertiesEditionComponent extends SinglePartPropertiesEditingComponent {

	/**
	 * @generated
	 */
	
	public static String BASE_PART = "Base"; //$NON-NLS-1$

	
	/**
	 * Settings for defaultStartPoint EObjectFlatComboViewer
	 */
	private	EObjectFlatComboSettings defaultStartPointSettings;
	
	/**
	 * Settings for startPoints ReferencesTable
	 */
	private	ReferencesTableSettings startPointsSettings;
	
	/**
	 * Settings for properties ReferencesTable
	 */
	protected ReferencesTableSettings propertiesSettings;
	
	/**
	 * Settings for defaultLib ReferencesTable
	 */
	private	ReferencesTableSettings defaultLibSettings;
	
	/**
	 * Settings for defaultSuperImposed ReferencesTable
	 */
	private	ReferencesTableSettings defaultSuperImposedSettings;
	
	/**
	 * Default constructor
	 * @generated
	 */
	public CallsPropertiesEditionComponent(PropertiesEditingContext editingContext, EObject calls, String editing_mode) {
    super(editingContext, calls, editing_mode);
    parts = new String[] { BASE_PART };
    repositoryKey = CallchainViewsRepository.class;
    partKey = CallchainViewsRepository.Calls.class;
  }

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.eclipse.emf.eef.runtime.api.component.IPropertiesEditionComponent#initPart(java.lang.Object, int, org.eclipse.emf.ecore.EObject, 
	 *      org.eclipse.emf.ecore.resource.ResourceSet)
	 * @generated
	 */
	public void initPart(Object key, int kind, EObject elt, ResourceSet allResource) {
    setInitializing(true);
    if (editingPart != null && key == partKey) {
      editingPart.setContext(elt, allResource);
      final Calls calls = (Calls)elt;
      final CallsPropertiesEditionPart basePart = (CallsPropertiesEditionPart)editingPart;
      // init values
      if (calls.getName() != null && isAccessible(CallchainViewsRepository.Calls.Properties.name))
        basePart.setName(EEFConverterUtil.convertToString(EcorePackage.eINSTANCE.getEString(), calls.getName()));
      
      if (isAccessible(CallchainViewsRepository.Calls.Properties.defaultStartPoint)) {
        // init part
        defaultStartPointSettings = new EObjectFlatComboSettings(calls, CallchainPackage.eINSTANCE.getCalls_DefaultStartPoint());
        basePart.initDefaultStartPoint(defaultStartPointSettings);
        // set the button mode
        basePart.setDefaultStartPointButtonMode(ButtonsModeEnum.BROWSE);
      }
      if (isAccessible(CallchainViewsRepository.Calls.Properties.startPoints)) {
        startPointsSettings = new ReferencesTableSettings(calls, CallchainPackage.eINSTANCE.getCalls_StartPoints());
        basePart.initStartPoints(startPointsSettings);
      }
      if (isAccessible(CallchainViewsRepository.Calls.Properties.defaultLib)) {
        defaultLibSettings = new ReferencesTableSettings(calls, CallchainPackage.eINSTANCE.getCalls_DefaultLib());
        basePart.initDefaultLib(defaultLibSettings);
      }
      if (isAccessible(CallchainViewsRepository.Calls.Properties.defaultSuperImposed)) {
        defaultSuperImposedSettings = new ReferencesTableSettings(calls, CallchainPackage.eINSTANCE.getCalls_DefaultSuperImposed());
        basePart.initDefaultSuperImposed(defaultSuperImposedSettings);
      }
      if (isAccessible(CallchainViewsRepository.Calls.Properties.logDefault)) {
        basePart.setLogDefault(calls.isLogDefault());
      }
      if (isAccessible(CallchainViewsRepository.Calls.Properties.properties_)) {
        propertiesSettings = new ReferencesTableSettings(calls, CallchainPackage.eINSTANCE.getCalls_Properties());
        basePart.initProperties(propertiesSettings);
      }
      // init filters
      
      basePart.addFilterToDefaultStartPoint(new ViewerFilter() {
      
      /**
       * {@inheritDoc}
       * 
       * @see org.eclipse.jface.viewers.ViewerFilter#select(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
       */
      public boolean select(Viewer viewer, Object parentElement, Object element) {
        return (element instanceof String && element.equals("")) || (element instanceof Callable); //$NON-NLS-1$ 
        }
      
      });
      
      basePart.addFilterToStartPoints(new ViewerFilter() {
      
        /**
         * {@inheritDoc}
         * 
         * @see org.eclipse.jface.viewers.ViewerFilter#select(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
         */
        public boolean select(Viewer viewer, Object parentElement, Object element) {
          if (element instanceof EObject)
            return (!basePart.isContainedInStartPointsTable((EObject)element));
          return element instanceof Resource;
        }
      
      });
      basePart.addFilterToStartPoints(new EObjectFilter(CallchainPackage.eINSTANCE.getCallable()));
      
      basePart.addFilterToDefaultLib(new ViewerFilter() {
      
        /**
         * {@inheritDoc}
         * 
         * @see org.eclipse.jface.viewers.ViewerFilter#select(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
         */
        public boolean select(Viewer viewer, Object parentElement, Object element) {
          if (element instanceof EObject)
            return (!basePart.isContainedInDefaultLibTable((EObject)element));
          return element instanceof Resource;
        }
      
      });
      basePart.addFilterToDefaultLib(new EObjectFilter(CallchainPackage.eINSTANCE.getAtlLibary()));
      
      basePart.addFilterToDefaultSuperImposed(new ViewerFilter() {
      
        /**
         * {@inheritDoc}
         * 
         * @see org.eclipse.jface.viewers.ViewerFilter#select(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
         */
        public boolean select(Viewer viewer, Object parentElement, Object element) {
          if (element instanceof EObject)
            return (!basePart.isContainedInDefaultSuperImposedTable((EObject)element));
          return element instanceof Resource;
        }
      
      });
      basePart.addFilterToDefaultSuperImposed(new EObjectFilter(CallchainPackage.eINSTANCE.getAtlRes()));
      
      
      basePart.addFilterToProperties(new ViewerFilter() {
      
          /**
           * {@inheritDoc}
           * 
           * @see org.eclipse.jface.viewers.ViewerFilter#select(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
           */
          public boolean select(Viewer viewer, Object parentElement, Object element) {
            return (element instanceof String && element.equals("")) || (element instanceof Property); //$NON-NLS-1$ 
          }
      
      });
      
      // init values for referenced views
      
      // init filters for referenced views
      
    }
    setInitializing(false);
  }









	/**
   * {@inheritDoc}
   * @see org.eclipse.emf.eef.runtime.impl.components.StandardPropertiesEditionComponent#associatedFeature(java.lang.Object)
   */
  public EStructuralFeature associatedFeature(Object editorKey) {
    if (editorKey == CallchainViewsRepository.Calls.Properties.name) {
      return CallchainPackage.eINSTANCE.getCalls_Name();
    }
    if (editorKey == CallchainViewsRepository.Calls.Properties.defaultStartPoint) {
      return CallchainPackage.eINSTANCE.getCalls_DefaultStartPoint();
    }
    if (editorKey == CallchainViewsRepository.Calls.Properties.startPoints) {
      return CallchainPackage.eINSTANCE.getCalls_StartPoints();
    }
    if (editorKey == CallchainViewsRepository.Calls.Properties.defaultLib) {
      return CallchainPackage.eINSTANCE.getCalls_DefaultLib();
    }
    if (editorKey == CallchainViewsRepository.Calls.Properties.defaultSuperImposed) {
      return CallchainPackage.eINSTANCE.getCalls_DefaultSuperImposed();
    }
    if (editorKey == CallchainViewsRepository.Calls.Properties.logDefault) {
      return CallchainPackage.eINSTANCE.getCalls_LogDefault();
    }
    if (editorKey == CallchainViewsRepository.Calls.Properties.properties_) {
      return CallchainPackage.eINSTANCE.getCalls_Properties();
    }
    return super.associatedFeature(editorKey);
  }

  /**
	 * {@inheritDoc}
	 * @see org.eclipse.emf.eef.runtime.impl.components.StandardPropertiesEditionComponent#updateSemanticModel(org.eclipse.emf.eef.runtime.api.notify.IPropertiesEditionEvent)
	 * @generated
	 */
	public void updateSemanticModel(final IPropertiesEditionEvent event) {
    Calls calls = (Calls)semanticObject;
    if (CallchainViewsRepository.Calls.Properties.name == event.getAffectedEditor()) {
      calls.setName((java.lang.String)EEFConverterUtil.createFromString(EcorePackage.eINSTANCE.getEString(), (String)event.getNewValue()));
    }
    if (CallchainViewsRepository.Calls.Properties.defaultStartPoint == event.getAffectedEditor()) {
      if (event.getKind() == PropertiesEditionEvent.SET) {
        defaultStartPointSettings.setToReference((Callable)event.getNewValue());
      } else if (event.getKind() == PropertiesEditionEvent.ADD) {
        EReferencePropertiesEditionContext context = new EReferencePropertiesEditionContext(editingContext, this, defaultStartPointSettings, editingContext.getAdapterFactory());
        PropertiesEditingProvider provider = (PropertiesEditingProvider)editingContext.getAdapterFactory().adapt(semanticObject, PropertiesEditingProvider.class);
        if (provider != null) {
          PropertiesEditingPolicy policy = provider.getPolicy(context);
          if (policy instanceof CreateEditingPolicy) {
            policy.execute();
          }
        }
      }
    }
    if (CallchainViewsRepository.Calls.Properties.startPoints == event.getAffectedEditor()) {
      if (event.getKind() == PropertiesEditionEvent.ADD) {
        if (event.getNewValue() instanceof Callable) {
          startPointsSettings.addToReference((EObject) event.getNewValue());
        }
      } else if (event.getKind() == PropertiesEditionEvent.REMOVE) {
        startPointsSettings.removeFromReference((EObject) event.getNewValue());
      } else if (event.getKind() == PropertiesEditionEvent.MOVE) {
        startPointsSettings.move(event.getNewIndex(), (Callable) event.getNewValue());
      }
    }
    if (CallchainViewsRepository.Calls.Properties.defaultLib == event.getAffectedEditor()) {
      if (event.getKind() == PropertiesEditionEvent.ADD) {
        if (event.getNewValue() instanceof AtlLibary) {
          defaultLibSettings.addToReference((EObject) event.getNewValue());
        }
      } else if (event.getKind() == PropertiesEditionEvent.REMOVE) {
        defaultLibSettings.removeFromReference((EObject) event.getNewValue());
      } else if (event.getKind() == PropertiesEditionEvent.MOVE) {
        defaultLibSettings.move(event.getNewIndex(), (AtlLibary) event.getNewValue());
      }
    }
    if (CallchainViewsRepository.Calls.Properties.defaultSuperImposed == event.getAffectedEditor()) {
      if (event.getKind() == PropertiesEditionEvent.ADD) {
        if (event.getNewValue() instanceof AtlRes) {
          defaultSuperImposedSettings.addToReference((EObject) event.getNewValue());
        }
      } else if (event.getKind() == PropertiesEditionEvent.REMOVE) {
        defaultSuperImposedSettings.removeFromReference((EObject) event.getNewValue());
      } else if (event.getKind() == PropertiesEditionEvent.MOVE) {
        defaultSuperImposedSettings.move(event.getNewIndex(), (AtlRes) event.getNewValue());
      }
    }
    if (CallchainViewsRepository.Calls.Properties.logDefault == event.getAffectedEditor()) {
      calls.setLogDefault((Boolean)event.getNewValue());
    }
    if (CallchainViewsRepository.Calls.Properties.properties_ == event.getAffectedEditor()) {
      if (event.getKind() == PropertiesEditionEvent.ADD) {
        EReferencePropertiesEditionContext context = new EReferencePropertiesEditionContext(editingContext, this, propertiesSettings, editingContext.getAdapterFactory());
        PropertiesEditingProvider provider = (PropertiesEditingProvider)editingContext.getAdapterFactory().adapt(semanticObject, PropertiesEditingProvider.class);
        if (provider != null) {
          PropertiesEditingPolicy policy = provider.getPolicy(context);
          if (policy instanceof CreateEditingPolicy) {
            policy.execute();
          }
        }
      } else if (event.getKind() == PropertiesEditionEvent.EDIT) {
        EObjectPropertiesEditionContext context = new EObjectPropertiesEditionContext(editingContext, this, (EObject) event.getNewValue(), editingContext.getAdapterFactory());
        PropertiesEditingProvider provider = (PropertiesEditingProvider)editingContext.getAdapterFactory().adapt((EObject) event.getNewValue(), PropertiesEditingProvider.class);
        if (provider != null) {
          PropertiesEditingPolicy editionPolicy = provider.getPolicy(context);
          if (editionPolicy != null) {
            editionPolicy.execute();
          }
        }
      } else if (event.getKind() == PropertiesEditionEvent.REMOVE) {
        propertiesSettings.removeFromReference((EObject) event.getNewValue());
      } else if (event.getKind() == PropertiesEditionEvent.MOVE) {
        propertiesSettings.move(event.getNewIndex(), (Property) event.getNewValue());
      }
    }
  }

	/**
	 * {@inheritDoc}
	 * @see org.eclipse.emf.eef.runtime.impl.components.StandardPropertiesEditionComponent#updatePart(org.eclipse.emf.common.notify.Notification)
	 */
	public void updatePart(Notification msg) {
		if (editingPart.isVisible()) {	
			CallsPropertiesEditionPart basePart = (CallsPropertiesEditionPart)editingPart;
			if (CallchainPackage.eINSTANCE.getCalls_DefaultStartPoint().equals(msg.getFeature()) && basePart != null && isAccessible(CallchainViewsRepository.Calls.Properties.defaultStartPoint))
				basePart.setDefaultStartPoint((EObject)msg.getNewValue());
			if (CallchainPackage.eINSTANCE.getCalls_StartPoints().equals(msg.getFeature())  && isAccessible(CallchainViewsRepository.Calls.Properties.startPoints))
				basePart.updateStartPoints();
			if (CallchainPackage.eINSTANCE.getCalls_Properties().equals(msg.getFeature()) && isAccessible(CallchainViewsRepository.Calls.Properties.properties_))
				basePart.updateProperties();
			if (CallchainPackage.eINSTANCE.getCalls_DefaultLib().equals(msg.getFeature())  && isAccessible(CallchainViewsRepository.Calls.Properties.defaultLib))
				basePart.updateDefaultLib();
			if (CallchainPackage.eINSTANCE.getCalls_DefaultSuperImposed().equals(msg.getFeature())  && isAccessible(CallchainViewsRepository.Calls.Properties.defaultSuperImposed))
				basePart.updateDefaultSuperImposed();
			if (CallchainPackage.eINSTANCE.getCalls_LogDefault().equals(msg.getFeature()) && basePart != null && isAccessible(CallchainViewsRepository.Calls.Properties.logDefault))
				basePart.setLogDefault((Boolean)msg.getNewValue());
			
			
		}
	}


	/**
   * {@inheritDoc}
   * 
   * @see org.eclipse.emf.eef.runtime.impl.components.StandardPropertiesEditionComponent#isRequired(java.lang.Object, int)
   * @generated
   */
  public boolean isRequired(Object key, int kind) {
    return key == CallchainViewsRepository.Calls.Properties.name;
  }

  /**
	 * {@inheritDoc}
	 * 
	 * @see org.eclipse.emf.eef.runtime.api.component.IPropertiesEditionComponent#validateValue(org.eclipse.emf.eef.runtime.api.notify.IPropertiesEditionEvent)
	 * @generated
	 */
	public Diagnostic validateValue(IPropertiesEditionEvent event) {
    Diagnostic ret = Diagnostic.OK_INSTANCE;
    if (event.getNewValue() != null) {
      try {
        if (CallchainViewsRepository.Calls.Properties.name == event.getAffectedEditor()) {
          Object newValue = event.getNewValue();
          if (newValue instanceof String) {
            newValue = EcoreUtil.createFromString(CallchainPackage.eINSTANCE.getCalls_Name().getEAttributeType(), (String)newValue);
          }
          ret = Diagnostician.INSTANCE.validate(CallchainPackage.eINSTANCE.getCalls_Name().getEAttributeType(), newValue);
        }
        if (CallchainViewsRepository.Calls.Properties.logDefault == event.getAffectedEditor()) {
          Object newValue = event.getNewValue();
          if (newValue instanceof String) {
            newValue = EcoreUtil.createFromString(CallchainPackage.eINSTANCE.getCalls_LogDefault().getEAttributeType(), (String)newValue);
          }
          ret = Diagnostician.INSTANCE.validate(CallchainPackage.eINSTANCE.getCalls_LogDefault().getEAttributeType(), newValue);
        }
      } catch (IllegalArgumentException iae) {
        ret = BasicDiagnostic.toDiagnostic(iae);
      } catch (WrappedException we) {
        ret = BasicDiagnostic.toDiagnostic(we);
      }
    }
    return ret;
  }

}
