package de.urszeidler.eclipse.callchain.diagram.edit.policies;

import java.util.Collections;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.gmf.runtime.common.core.command.ICommand;
import org.eclipse.gmf.runtime.common.core.command.ICompositeCommand;
import org.eclipse.gmf.runtime.diagram.core.commands.DeleteCommand;
import org.eclipse.gmf.runtime.diagram.ui.commands.CommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editpolicies.SemanticEditPolicy;
import org.eclipse.gmf.runtime.emf.commands.core.command.CompositeTransactionalCommand;
import org.eclipse.gmf.runtime.emf.type.core.IElementType;
import org.eclipse.gmf.runtime.emf.type.core.requests.ConfigureRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateElementRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyReferenceRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DuplicateElementsRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.GetEditContextRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.IEditCommandRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.MoveRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientReferenceRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.SetRequest;
import org.eclipse.gmf.runtime.notation.View;

import de.urszeidler.eclipse.callchain.Artifact;
import de.urszeidler.eclipse.callchain.AtlLibary;
import de.urszeidler.eclipse.callchain.AtlRes;
import de.urszeidler.eclipse.callchain.Call;
import de.urszeidler.eclipse.callchain.Callable;
import de.urszeidler.eclipse.callchain.CallchainPackage;
import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.Comment;
import de.urszeidler.eclipse.callchain.Documentable;
import de.urszeidler.eclipse.callchain.Generator;
import de.urszeidler.eclipse.callchain.MMAlias;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.ModelAlias;
import de.urszeidler.eclipse.callchain.NamedSwitch;
import de.urszeidler.eclipse.callchain.PredicateSwitch;
import de.urszeidler.eclipse.callchain.Switch;
import de.urszeidler.eclipse.callchain.Transition;
import de.urszeidler.eclipse.callchain.diagram.edit.helpers.CallchainBaseEditHelper;
import de.urszeidler.eclipse.callchain.diagram.expressions.CallchainOCLFactory;
import de.urszeidler.eclipse.callchain.diagram.part.CallchainDiagramEditorPlugin;
import de.urszeidler.eclipse.callchain.diagram.part.CallchainVisualIDRegistry;
import de.urszeidler.eclipse.callchain.diagram.providers.CallchainElementTypes;

/**
 * @generated
 */
public class CallchainBaseItemSemanticEditPolicy extends SemanticEditPolicy {

	/**
	 * Extended request data key to hold editpart visual id.
	 * 
	 * @generated
	 */
	public static final String VISUAL_ID_KEY = "visual_id"; //$NON-NLS-1$

	/**
	 * @generated
	 */
	private final IElementType myElementType;

	/**
	 * @generated
	 */
	protected CallchainBaseItemSemanticEditPolicy(IElementType elementType) {
		myElementType = elementType;
	}

	/**
	 * Extended request data key to hold editpart visual id.
	 * Add visual id of edited editpart to extended data of the request
	 * so command switch can decide what kind of diagram element is being edited.
	 * It is done in those cases when it's not possible to deduce diagram
	 * element kind from domain element.
	 * 
	 * @generated
	 */
	public Command getCommand(Request request) {
		if (request instanceof ReconnectRequest) {
			Object view = ((ReconnectRequest) request).getConnectionEditPart().getModel();
			if (view instanceof View) {
				Integer id = new Integer(CallchainVisualIDRegistry.getVisualID((View) view));
				request.getExtendedData().put(VISUAL_ID_KEY, id);
			}
		}
		return super.getCommand(request);
	}

	/**
	 * Returns visual id from request parameters.
	 * 
	 * @generated
	 */
	protected int getVisualID(IEditCommandRequest request) {
		Object id = request.getParameter(VISUAL_ID_KEY);
		return id instanceof Integer ? ((Integer) id).intValue() : -1;
	}

	/**
	 * @generated
	 */
	protected Command getSemanticCommand(IEditCommandRequest request) {
		IEditCommandRequest completedRequest = completeRequest(request);
		Command semanticCommand = getSemanticCommandSwitch(completedRequest);
		semanticCommand = getEditHelperCommand(completedRequest, semanticCommand);
		if (completedRequest instanceof DestroyRequest) {
			DestroyRequest destroyRequest = (DestroyRequest) completedRequest;
			return shouldProceed(destroyRequest) ? addDeleteViewCommand(semanticCommand, destroyRequest) : null;
		}
		return semanticCommand;
	}

	/**
	 * @generated
	 */
	protected Command addDeleteViewCommand(Command mainCommand, DestroyRequest completedRequest) {
		Command deleteViewCommand = getGEFWrapper(new DeleteCommand(getEditingDomain(), (View) getHost().getModel()));
		return mainCommand == null ? deleteViewCommand : mainCommand.chain(deleteViewCommand);
	}

	/**
	 * @generated
	 */
	private Command getEditHelperCommand(IEditCommandRequest request, Command editPolicyCommand) {
		if (editPolicyCommand != null) {
			ICommand command = editPolicyCommand instanceof ICommandProxy ? ((ICommandProxy) editPolicyCommand)
					.getICommand() : new CommandProxy(editPolicyCommand);
			request.setParameter(CallchainBaseEditHelper.EDIT_POLICY_COMMAND, command);
		}
		IElementType requestContextElementType = getContextElementType(request);
		request.setParameter(CallchainBaseEditHelper.CONTEXT_ELEMENT_TYPE, requestContextElementType);
		ICommand command = requestContextElementType.getEditCommand(request);
		request.setParameter(CallchainBaseEditHelper.EDIT_POLICY_COMMAND, null);
		request.setParameter(CallchainBaseEditHelper.CONTEXT_ELEMENT_TYPE, null);
		if (command != null) {
			if (!(command instanceof CompositeTransactionalCommand)) {
				command = new CompositeTransactionalCommand(getEditingDomain(), command.getLabel()).compose(command);
			}
			return new ICommandProxy(command);
		}
		return editPolicyCommand;
	}

	/**
	 * @generated
	 */
	private IElementType getContextElementType(IEditCommandRequest request) {
		IElementType requestContextElementType = CallchainElementTypes.getElementType(getVisualID(request));
		return requestContextElementType != null ? requestContextElementType : myElementType;
	}

	/**
	 * @generated
	 */
	protected Command getSemanticCommandSwitch(IEditCommandRequest req) {
		if (req instanceof CreateRelationshipRequest) {
			return getCreateRelationshipCommand((CreateRelationshipRequest) req);
		} else if (req instanceof CreateElementRequest) {
			return getCreateCommand((CreateElementRequest) req);
		} else if (req instanceof ConfigureRequest) {
			return getConfigureCommand((ConfigureRequest) req);
		} else if (req instanceof DestroyElementRequest) {
			return getDestroyElementCommand((DestroyElementRequest) req);
		} else if (req instanceof DestroyReferenceRequest) {
			return getDestroyReferenceCommand((DestroyReferenceRequest) req);
		} else if (req instanceof DuplicateElementsRequest) {
			return getDuplicateCommand((DuplicateElementsRequest) req);
		} else if (req instanceof GetEditContextRequest) {
			return getEditContextCommand((GetEditContextRequest) req);
		} else if (req instanceof MoveRequest) {
			return getMoveCommand((MoveRequest) req);
		} else if (req instanceof ReorientReferenceRelationshipRequest) {
			return getReorientReferenceRelationshipCommand((ReorientReferenceRelationshipRequest) req);
		} else if (req instanceof ReorientRelationshipRequest) {
			return getReorientRelationshipCommand((ReorientRelationshipRequest) req);
		} else if (req instanceof SetRequest) {
			return getSetCommand((SetRequest) req);
		}
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getConfigureCommand(ConfigureRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getCreateRelationshipCommand(CreateRelationshipRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getCreateCommand(CreateElementRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getSetCommand(SetRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getEditContextCommand(GetEditContextRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getDestroyElementCommand(DestroyElementRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getDestroyReferenceCommand(DestroyReferenceRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getDuplicateCommand(DuplicateElementsRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getMoveCommand(MoveRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getReorientReferenceRelationshipCommand(ReorientReferenceRelationshipRequest req) {
		return UnexecutableCommand.INSTANCE;
	}

	/**
	 * @generated
	 */
	protected Command getReorientRelationshipCommand(ReorientRelationshipRequest req) {
		return UnexecutableCommand.INSTANCE;
	}

	/**
	 * @generated
	 */
	protected final Command getGEFWrapper(ICommand cmd) {
		return new ICommandProxy(cmd);
	}

	/**
	 * Returns editing domain from the host edit part.
	 * 
	 * @generated
	 */
	protected TransactionalEditingDomain getEditingDomain() {
		return ((IGraphicalEditPart) getHost()).getEditingDomain();
	}

	/**
	 * Clean all shortcuts to the host element from the same diagram
	 * @generated
	 */
	protected void addDestroyShortcutsCommand(ICompositeCommand cmd, View view) {
		assert view.getEAnnotation("Shortcut") == null; //$NON-NLS-1$
		for (Iterator it = view.getDiagram().getChildren().iterator(); it.hasNext();) {
			View nextView = (View) it.next();
			if (nextView.getEAnnotation("Shortcut") == null || !nextView.isSetElement() || nextView.getElement() != view.getElement()) { //$NON-NLS-1$
				continue;
			}
			cmd.add(new DeleteCommand(getEditingDomain(), nextView));
		}
	}

	/**
	 * @generated
	 */
	public static LinkConstraints getLinkConstraints() {
		LinkConstraints cached = CallchainDiagramEditorPlugin.getInstance().getLinkConstraints();
		if (cached == null) {
			CallchainDiagramEditorPlugin.getInstance().setLinkConstraints(cached = new LinkConstraints());
		}
		return cached;
	}

	/**
	 * @generated
	 */
	public static class LinkConstraints {
		/**
		 * @generated
		 */
		LinkConstraints() {
			// use static method #getLinkConstraints() to access instance
		}

		/**
		 * @generated
		 */
		public boolean canCreateModelAlias_4016(Call source, Model target) {
			return canExistModelAlias_4016(null, source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateMMAlias_4017(Calls source, Model target) {
			return canExistMMAlias_4017(null, source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateTransition_4029(NamedSwitch source, Callable target) {
			return canExistTransition_4029(null, source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateModelMm_4018(Model source, de.urszeidler.eclipse.callchain.MM target) {
			if (source != null) {
				if (source.getMm() != null) {
					return false;
				}
			}

			return canExistModelMm_4018(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateAtlResSuperimpose_4019(AtlRes source, AtlRes target) {
			if (source != null) {
				if (source.getSuperimpose().contains(target)) {
					return false;
				}
			}

			return canExistAtlResSuperimpose_4019(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateCallIn_4020(Call source, Model target) {
			if (source != null) {
				if (source.getIn().contains(target)) {
					return false;
				}
			}

			return canExistCallIn_4020(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateCallOut_4021(Call source, Model target) {
			if (source != null) {
				if (source.getOut().contains(target)) {
					return false;
				}
			}

			return canExistCallOut_4021(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateCallRes_4022(Call source, AtlRes target) {
			if (source != null) {
				if (source.getRes() != null) {
					return false;
				}
			}

			return canExistCallRes_4022(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateCallLibarys_4023(Call source, AtlLibary target) {
			if (source != null) {
				if (source.getLibarys().contains(target)) {
					return false;
				}
			}

			return canExistCallLibarys_4023(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateCallArtifact_4024(Call source, Artifact target) {
			if (source != null) {
				if (source.getArtifact() != null) {
					return false;
				}
			}

			return canExistCallArtifact_4024(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateCommentDocumentedElement_4025(Comment source, Documentable target) {
			if (source != null) {
				if (source.getDocumentedElement().contains(target)) {
					return false;
				}
			}

			return canExistCommentDocumentedElement_4025(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreatePredicateSwitchTrueCallable_4030(PredicateSwitch source, Callable target) {
			if (source != null) {
				if (source.getTrueCallable() != null) {
					return false;
				}
			}

			return canExistPredicateSwitchTrueCallable_4030(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreatePredicateSwitchFalseCallable_4031(PredicateSwitch source, Callable target) {
			if (source != null) {
				if (source.getFalseCallable() != null) {
					return false;
				}
			}

			return canExistPredicateSwitchFalseCallable_4031(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateGeneratorUses_4026(Generator source, Artifact target) {
			if (source != null) {
				if (source.getUses().contains(target)) {
					return false;
				}
			}

			return canExistGeneratorUses_4026(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateGeneratorProduce_4027(Generator source, Artifact target) {
			if (source != null) {
				if (source.getProduce().contains(target)) {
					return false;
				}
			}

			return canExistGeneratorProduce_4027(source, target);
		}

		/**
		 * @generated not
		 */
		public boolean canCreateCallableNext_4028(Callable source, Callable target) {
			if (source != null) {
				if (source instanceof Switch) {
					return false;
				}
				if (source.getNext() != null) {
					return false;
				}
			}

			return canExistCallableNext_4028(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canCreateNamedSwitchDefaultCall_4032(NamedSwitch source, Callable target) {
			if (source != null) {
				if (source.getDefaultCall() != null) {
					return false;
				}
			}

			return canExistNamedSwitchDefaultCall_4032(source, target);
		}

		/**
		 * @generated
		 */
		public boolean canExistModelAlias_4016(ModelAlias linkInstance, Call source, Model target) {
			try {
				if (source == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getModel()); //$NON-NLS-1$
					Object sourceVal = CallchainOCLFactory.getExpression(12, CallchainPackage.eINSTANCE.getCall(), env)
							.evaluate(source, Collections.singletonMap("oppositeEnd", target)); //$NON-NLS-1$
					if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				if (target == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getCall()); //$NON-NLS-1$
					Object targetVal = CallchainOCLFactory
							.getExpression(13, CallchainPackage.eINSTANCE.getModel(), env).evaluate(target,
									Collections.singletonMap("oppositeEnd", source)); //$NON-NLS-1$
					if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public boolean canExistMMAlias_4017(MMAlias linkInstance, Calls source, Model target) {
			try {
				if (source == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getModel()); //$NON-NLS-1$
					Object sourceVal = CallchainOCLFactory
							.getExpression(14, CallchainPackage.eINSTANCE.getCalls(), env).evaluate(source,
									Collections.singletonMap("oppositeEnd", target)); //$NON-NLS-1$
					if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				if (target == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getCalls()); //$NON-NLS-1$
					Object targetVal = CallchainOCLFactory
							.getExpression(15, CallchainPackage.eINSTANCE.getModel(), env).evaluate(target,
									Collections.singletonMap("oppositeEnd", source)); //$NON-NLS-1$
					if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public boolean canExistTransition_4029(Transition linkInstance, NamedSwitch source, Callable target) {
			try {
				if (source == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getCallable()); //$NON-NLS-1$
					Object sourceVal = CallchainOCLFactory.getExpression(16,
							CallchainPackage.eINSTANCE.getNamedSwitch(), env).evaluate(source,
							Collections.singletonMap("oppositeEnd", target)); //$NON-NLS-1$
					if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				if (target == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getNamedSwitch()); //$NON-NLS-1$
					Object targetVal = CallchainOCLFactory.getExpression(17, CallchainPackage.eINSTANCE.getCallable(),
							env).evaluate(target, Collections.singletonMap("oppositeEnd", source)); //$NON-NLS-1$
					if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public boolean canExistModelMm_4018(Model source, de.urszeidler.eclipse.callchain.MM target) {
			return true;
		}

		/**
		 * @generated
		 */
		public boolean canExistAtlResSuperimpose_4019(AtlRes source, AtlRes target) {
			try {
				if (source == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getAtlRes()); //$NON-NLS-1$
					Object sourceVal = CallchainOCLFactory.getExpression(18, CallchainPackage.eINSTANCE.getAtlRes(),
							env).evaluate(source, Collections.singletonMap("oppositeEnd", target)); //$NON-NLS-1$
					if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				if (target == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getAtlRes()); //$NON-NLS-1$
					Object targetVal = CallchainOCLFactory.getExpression(19, CallchainPackage.eINSTANCE.getAtlRes(),
							env).evaluate(target, Collections.singletonMap("oppositeEnd", source)); //$NON-NLS-1$
					if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public boolean canExistCallIn_4020(Call source, Model target) {
			try {
				if (source == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getModel()); //$NON-NLS-1$
					Object sourceVal = CallchainOCLFactory.getExpression(20, CallchainPackage.eINSTANCE.getCall(), env)
							.evaluate(source, Collections.singletonMap("oppositeEnd", target)); //$NON-NLS-1$
					if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				if (target == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getCall()); //$NON-NLS-1$
					Object targetVal = CallchainOCLFactory
							.getExpression(21, CallchainPackage.eINSTANCE.getModel(), env).evaluate(target,
									Collections.singletonMap("oppositeEnd", source)); //$NON-NLS-1$
					if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public boolean canExistCallOut_4021(Call source, Model target) {
			return true;
		}

		/**
		 * @generated
		 */
		public boolean canExistCallRes_4022(Call source, AtlRes target) {
			return true;
		}

		/**
		 * @generated
		 */
		public boolean canExistCallLibarys_4023(Call source, AtlLibary target) {
			return true;
		}

		/**
		 * @generated
		 */
		public boolean canExistCallArtifact_4024(Call source, Artifact target) {
			return true;
		}

		/**
		 * @generated
		 */
		public boolean canExistCommentDocumentedElement_4025(Comment source, Documentable target) {
			return true;
		}

		/**
		 * @generated
		 */
		public boolean canExistPredicateSwitchTrueCallable_4030(PredicateSwitch source, Callable target) {
			try {
				if (source == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getCallable()); //$NON-NLS-1$
					Object sourceVal = CallchainOCLFactory.getExpression(22,
							CallchainPackage.eINSTANCE.getPredicateSwitch(), env).evaluate(source,
							Collections.singletonMap("oppositeEnd", target)); //$NON-NLS-1$
					if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				if (target == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getPredicateSwitch()); //$NON-NLS-1$
					Object targetVal = CallchainOCLFactory.getExpression(23, CallchainPackage.eINSTANCE.getCallable(),
							env).evaluate(target, Collections.singletonMap("oppositeEnd", source)); //$NON-NLS-1$
					if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public boolean canExistPredicateSwitchFalseCallable_4031(PredicateSwitch source, Callable target) {
			try {
				if (source == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getCallable()); //$NON-NLS-1$
					Object sourceVal = CallchainOCLFactory.getExpression(24,
							CallchainPackage.eINSTANCE.getPredicateSwitch(), env).evaluate(source,
							Collections.singletonMap("oppositeEnd", target)); //$NON-NLS-1$
					if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				if (target == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getPredicateSwitch()); //$NON-NLS-1$
					Object targetVal = CallchainOCLFactory.getExpression(25, CallchainPackage.eINSTANCE.getCallable(),
							env).evaluate(target, Collections.singletonMap("oppositeEnd", source)); //$NON-NLS-1$
					if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public boolean canExistGeneratorUses_4026(Generator source, Artifact target) {
			try {
				if (source == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getArtifact()); //$NON-NLS-1$
					Object sourceVal = CallchainOCLFactory.getExpression(26, CallchainPackage.eINSTANCE.getGenerator(),
							env).evaluate(source, Collections.singletonMap("oppositeEnd", target)); //$NON-NLS-1$
					if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				if (target == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getGenerator()); //$NON-NLS-1$
					Object targetVal = CallchainOCLFactory.getExpression(27, CallchainPackage.eINSTANCE.getArtifact(),
							env).evaluate(target, Collections.singletonMap("oppositeEnd", source)); //$NON-NLS-1$
					if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public boolean canExistGeneratorProduce_4027(Generator source, Artifact target) {
			try {
				if (source == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getArtifact()); //$NON-NLS-1$
					Object sourceVal = CallchainOCLFactory.getExpression(28, CallchainPackage.eINSTANCE.getGenerator(),
							env).evaluate(source, Collections.singletonMap("oppositeEnd", target)); //$NON-NLS-1$
					if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				if (target == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getGenerator()); //$NON-NLS-1$
					Object targetVal = CallchainOCLFactory.getExpression(29, CallchainPackage.eINSTANCE.getArtifact(),
							env).evaluate(target, Collections.singletonMap("oppositeEnd", source)); //$NON-NLS-1$
					if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public boolean canExistCallableNext_4028(Callable source, Callable target) {
			try {
				if (source == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getCallable()); //$NON-NLS-1$
					Object sourceVal = CallchainOCLFactory.getExpression(30, CallchainPackage.eINSTANCE.getCallable(),
							env).evaluate(source, Collections.singletonMap("oppositeEnd", target)); //$NON-NLS-1$
					if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				if (target == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getCallable()); //$NON-NLS-1$
					Object targetVal = CallchainOCLFactory.getExpression(31, CallchainPackage.eINSTANCE.getCallable(),
							env).evaluate(target, Collections.singletonMap("oppositeEnd", source)); //$NON-NLS-1$
					if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public boolean canExistNamedSwitchDefaultCall_4032(NamedSwitch source, Callable target) {
			try {
				if (source == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getCallable()); //$NON-NLS-1$
					Object sourceVal = CallchainOCLFactory.getExpression(32,
							CallchainPackage.eINSTANCE.getNamedSwitch(), env).evaluate(source,
							Collections.singletonMap("oppositeEnd", target)); //$NON-NLS-1$
					if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				if (target == null) {
					return true;
				} else {
					Map<String, EClassifier> env = Collections.<String, EClassifier> singletonMap(
							"oppositeEnd", CallchainPackage.eINSTANCE.getNamedSwitch()); //$NON-NLS-1$
					Object targetVal = CallchainOCLFactory.getExpression(33, CallchainPackage.eINSTANCE.getCallable(),
							env).evaluate(target, Collections.singletonMap("oppositeEnd", source)); //$NON-NLS-1$
					if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
						return false;
					} // else fall-through
				}
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

	}

}
