/**
 * 
 */
package de.urszeidler.eclipse.callchain.executors;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;

import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;
import de.urszeidler.eclipse.callchain.provider.CallchainEditPlugin;

/**
 * @author urs
 *
 */
public class GenericExecutorStep implements ExecutorStep {
	private GeneratorExecutable executable;
	private Generic_Generator generator;

	public GenericExecutorStep(GeneratorExecutable executable,
			Generic_Generator generator) {
		super();
		this.executable = executable;
		this.generator = generator;
		this.executable.configure(generator);
		
	}

	/* (non-Javadoc)
	 * @see de.urszeidler.eclipse.callchain.executors.ExecutorStep#generate(java.lang.Object)
	 */
	@Override
	public void generate(Object monitor) {
		if(monitor==null)
			monitor= new NullProgressMonitor();
				
		try {
			executable.execute((IProgressMonitor) monitor);
		} catch (CoreException e) {
			CallchainEditPlugin.log("error while generating : "+getTaskName(), IStatus.ERROR, e);
		}

	}

	/* (non-Javadoc)
	 * @see de.urszeidler.eclipse.callchain.executors.ExecutorStep#getTaskName()
	 */
	@Override
	public String getTaskName() {
		if(executable==null)
			throw new NullPointerException("no executor set");
		if(executable.getName()==null)
			return executable.toString();
		
		return executable.getName();
	}

	@Override
	public Object getWrapedObject() {
		return generator;
	}

	@Override
	public boolean isRefreshWorkspace() {
		return generator.isRefreshWorkspace();
	}

}
