/*
 * (c) urs zeidler
 */
package de.urszeidler.eclipse.callchain.executors;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.m2m.atl.core.ui.ATLConsole;

import de.urszeidler.eclipse.callchain.Artifact;
import de.urszeidler.eclipse.callchain.AtlLibary;
import de.urszeidler.eclipse.callchain.AtlRes;
import de.urszeidler.eclipse.callchain.Call;
import de.urszeidler.eclipse.callchain.Callable;
import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.Eavaluateable;
import de.urszeidler.eclipse.callchain.ExternalCallable;
import de.urszeidler.eclipse.callchain.Generator;
import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.ModelAlias;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorDescriptor;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;
import de.urszeidler.eclipse.callchain.generatorservice.Generatorservice;
import de.urszeidler.eclipse.callchain.provider.CallchainEditPlugin;
import de.urszeidler.eclipse.callchain.util.CallchainSwitch;

/**
 * this one manages the transformation calls
 * 
 * @author urs
 * 
 */
public class MCcallchain {
	private final class SingleTransformationJob extends WorkspaceJob {
		private final Callable callableToStart;

		private SingleTransformationJob(String name, Callable callableToStart) {
			super(name);
			this.callableToStart = callableToStart;
		}

		@Override
		public IStatus runInWorkspace(IProgressMonitor monitor) throws CoreException {
			final IProgressMonitor monitor1 = monitor;

			Calls calls = (Calls) callableToStart.eContainer();
			try {
				calls.setSubtitutionMode(true);

				final CallchainSwitch<Object> swt = new CallchainSwitch<Object>() {
					@Override
					public Object caseGenerator(Generator object) {
						Object doSwitch = executorBuilder.doSwitch(object);
						if (doSwitch instanceof ExecutorStep) {
							ExecutorStep es = (ExecutorStep) doSwitch;
							es.generate(monitor1);
						} else if (doSwitch instanceof List) {
							List<ExecutorStep> l = (List<ExecutorStep>) doSwitch;
							for (ExecutorStep executorStep : l) {
								executorStep.generate(monitor1);
							}
						}
						return null;
					}

					@Override
					public Object caseCall(Call object) {
						launchCall(object, monitor1);
						return null;
					}
				};

				swt.doSwitch(callableToStart);
			} finally {
				calls.setSubtitutionMode(false);
			}

			return Status.OK_STATUS;
		}
	}

	private final class TransformationChainJob extends WorkspaceJob {
		private final Callable callableToStart;
		private final List<String> skipplist;

		private TransformationChainJob(String name, Callable callableToStart, List<String> skipplist) {
			super(name);
			this.callableToStart = callableToStart;
			this.skipplist = skipplist;
		}

		public IStatus runInWorkspace(IProgressMonitor monitor) {
			Calls calls = (Calls) callableToStart.eContainer();

			try {
				executorBuilder.getCalls().add(calls);
				calls.setSubtitutionMode(true);

				final List<ExecutorStep> buildExecutionList = buildExecutionList(callableToStart, skipplist);
				monitor.beginTask("Start transformation ", buildExecutionList.size());
				for (ExecutorStep executorStep : buildExecutionList) {
					if (monitor.isCanceled())
						break;

					monitor.subTask(executorStep.getTaskName());
					try {
						executorStep.generate(monitor);
						if (executorStep.isRefreshWorkspace()) {
							try {
								ResourcesPlugin.getWorkspace().getRoot().refreshLocal(2, monitor);
							} catch (CoreException e) {
								CallchainEditPlugin.log("Error while refreshing workspace", IStatus.ERROR, e);
							}
						}
					} finally {
						monitor.worked(1);
					}
				}

			} finally {
				setSubstitution(false);
				monitor.done();
			}
			return Status.OK_STATUS;
		}
	}

	// TODO : need a lot refactoring
	private class ExecutorBuilder extends CallchainSwitch<Object> {

		private int reusiveCounter;
		private Set<Calls> calls = new HashSet<Calls>();

		@Override
		public List<ExecutorStep> caseExternalCallable(ExternalCallable object) {
			Callable delegatedCallable = object.getDelegatedCallable();
			if (delegatedCallable == null)
				return null;
			
			reusiveCounter++;
			if (reusiveCounter > 20)
				return null;

			List<ExecutorStep> list = buildExecutionList(delegatedCallable, Collections.<String> emptyList());
				
			list.add(0, new StartExternalCallableStep(object));
			list.add(new StopExternalCallableStep(object));
			
			
			return list;
		}

		@Override
		public ExecutorStep caseCall(Call object) {
			reusiveCounter = 0;

			return new CallExecutorStep(object);
		}

		@Override
		public ExecutorStep caseGeneric_Generator(Generic_Generator object) {
			reusiveCounter = 0;

			GeneratorDescriptor descriptor = Generatorservice.getDefault().getGenerator(object.getId());
			if (descriptor != null) {
				GeneratorExecutable executor;
				try {
					executor = descriptor.createExecutable();
					return new GenericExecutorStep(executor, object);
				} catch (InstantiationException e) {
					CallchainEditPlugin
							.log("Error creating executable :" + descriptor.getClassName(), IStatus.ERROR, e);
				} catch (IllegalAccessException e) {
					CallchainEditPlugin
							.log("Error creating executable :" + descriptor.getClassName(), IStatus.ERROR, e);
				}
			}
			return null;
		}

		public Set<Calls> getCalls() {
			return calls;
		}
	}

	/**
	 * @author urs executes the atl call
	 */
	private class CallExecutorStep implements ExecutorStep {

		private final Call call;

		public CallExecutorStep(Call object) {
			this.call = object;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * de.urszeidler.eclipse.callchain.executors.ExecutorStep#generate(java
		 * .lang.Object)
		 */
		@Override
		public void generate(Object monitor) {
			launchCall(call, (IProgressMonitor) monitor);

		}

		@Override
		public String getTaskName() {
			return call.getName();
		}

		@Override
		public Object getWrapedObject() {
			return call;
		}

		@Override
		public boolean isRefreshWorkspace() {
			return call.isRefreshWorkspace();
		}

	}

	public static final String ATT_URI = "uri";
	private final List<Model> tmpModels = new ArrayList<Model>();
	private ExecutorBuilder executorBuilder = new ExecutorBuilder();
	private ATLConsole atlConsole;

	public MCcallchain() {
	}

	/**
	 * 
	 * @param callableToStart
	 * @param skipplist
	 * @return the list of executors to execute
	 */
	public List<ExecutorStep> buildExecutionList(final Callable callableToStart, final List<String> skipplist) {
		final List<ExecutorStep> list = new ArrayList<ExecutorStep>();
		Callable lcall = callableToStart;

		while (lcall != null) {
			if (!skipplist.contains(lcall.getName())) {
				Object doSwitch = null;
				if (lcall instanceof Eavaluateable) {
					Eavaluateable e = (Eavaluateable) lcall;
					if (e.evaluate())
						doSwitch = executorBuilder.doSwitch(lcall);
				} else {
					doSwitch = executorBuilder.doSwitch(lcall);
				}
				if (doSwitch instanceof ExecutorStep) {
					ExecutorStep es = (ExecutorStep) doSwitch;
					list.add(es);
				} else if (doSwitch instanceof List) {
					List<ExecutorStep> l = (List<ExecutorStep>) doSwitch;
					list.addAll(l);
				}
			}
			lcall = lcall.getNext();
		}
		return list;
	}

	/**
	 * 
	 * @param callableToStart
	 * @param skipplist
	 */
	public void executeTransformations(final Callable callableToStart, final List<String> skipplist) {
		executorBuilder.getCalls().clear();
		Job job = new TransformationChainJob("Start transformation :", callableToStart, skipplist);
		job.setUser(true);
		job.schedule();

		try {
			ResourcesPlugin.getWorkspace().getRoot().refreshLocal(2, null);
		} catch (CoreException e) {
			CallchainEditPlugin.log("Error while refreshing workspace", IStatus.ERROR, e);
		}

	}

	protected void setSubstitution(boolean subtitutionMode) {
		Set<Calls> calls = executorBuilder.getCalls();
		for (Calls calls2 : calls) {
			calls2.setSubtitutionMode(subtitutionMode);
		}

	}

	/**
	 * delete the tmp models
	 * 
	 * @param tmpModels2
	 */
	protected void deletetmpFiles(List<Model> tmpModels2) {
		IWorkspace wks = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot wksroot = wks.getRoot();

		for (Model model : tmpModels2) {
			IFile file = wksroot.getFile(new Path(model.getUri()));
			if (file.exists())
				try {
					CallchainEditPlugin.log("deleteting tmp Model :" + file.toString(), IStatus.OK);
					file.delete(true, null);
				} catch (CoreException e) {
					CallchainEditPlugin.log("Error while deleteting tmp Model :" + file.toString(), IStatus.WARNING, e);
				}
		}

	}

	/**
	 * 
	 * @param call
	 * @param pmon
	 */
	protected void launchCall(Call call, IProgressMonitor pmon) {
		// AtlLaunchConfigurationDelegate launchConfigurationDelegate = new
		// AtlLaunchConfigurationDelegate();
		if (atlConsole == null)
			atlConsole = ATLConsole.findConsole();
		String filePath = call.getRes().getUri();
		Map<String, String> libsFromConfig = collectLibs(call);
		Map<String, String> input = collectModel2MM(call.getIn());
		Map<String, String> output = collectModel2MM(call.getOut());
		Map<String, String> path = collectPath(call);
		Map<String, String> modelType = collectModelTypes(call);
		Map<String, String> modelHandler = collectModelHandler(call);
		String mode = "run";
		// boolean checkSameModel = !call.isAllowInterModelReferences();//
		// false;
		List<String> superimpose = collectSuperImposed(call);// Collections.EMPTY_LIST;
		// boolean continueAfterErrors = call.isContinueAfterErrors();
		Map<String, Object> options = new HashMap<String, Object>();

		collectTmpModels(call.getIn());
		if (call.getEnviorment().isLogDefault())
			CallchainEditPlugin.log("execute transformation :" + filePath.toString(), IStatus.INFO);

		// TODO need to check the drawbacks
		options.put("monitor", pmon); //$NON-NLS-1$
		// "Regular VM (with debugger)"
		// "EMF-specific VM"
		options.put("allowInterModelReferences", new Boolean(call.isAllowInterModelReferences()).toString());
		options.put("continueAfterError", new Boolean(call.isContinueAfterErrors()).toString());
		options.put("disableAttributeHelperCache", new Boolean(call.isDisableAttributeHelperCache()).toString());
		options.put("printExecutionTime", new Boolean(call.isPrintExecutionTime()).toString());
		options.put("profile", new Boolean(call.isProfile()).toString());
		options.put("showSummary", new Boolean(call.isShowSummary()).toString());

		if (call.getArtifact() != null) {
			if (call.getArtifact().isDerived() == false)
				options.put("OPTION_DERIVED", "false");
		}
		for (Model model : call.getOut()) {
			if (model.isDerived() == false)
				options.put("OPTION_DERIVED", "false");
		}

		try {
			// URL url = fileNameToURL(filePath);
			CallchainAtlLaunchConfigurationDelegate configurationDelegate = new CallchainAtlLaunchConfigurationDelegate();
			configurationDelegate.launch(mode, pmon, false, false, false, "", "Regular VM (with debugger)", filePath,
					input, output, path, libsFromConfig, superimpose, modelHandler, options);

			// // TODO : this need to be done in the executor
			// if (call.getArtifact() != null)
			// ResourcesPlugin.getWorkspace().getRoot().refreshLocal(2, null);

		} catch (Exception e) {
			CallchainEditPlugin.log("execute transformation :" + filePath.toString(), IStatus.ERROR, e);
		}
	}

	/**
	 * collect all temp models
	 * 
	 * @param list
	 */
	private void collectTmpModels(EList<Model> list) {
		for (Object o : list) {
			if (o instanceof Model && !(o instanceof ModelAlias)) {
				Model model = (Model) o;
				if (model.isIsTemporary())
					tmpModels.add(model);
			}// end of if (o instanceof Model )
		}
	}

	/**
	 * 
	 * @param call
	 * @return
	 */
	private List<String> collectSuperImposed(Call call) {
		ArrayList<String> list = new ArrayList<String>();
		AtlRes atlRes = call.getRes();
		if (atlRes != null) {
			EList<AtlRes> superimpose = atlRes.getSuperimpose();
			superimpose.addAll(call.getEnviorment().getDefaultSuperImposed());
			for (Object object : superimpose) {
				if (object instanceof AtlRes) {
					AtlRes satl = (AtlRes) object;
					String atlFile = satl.getUri();
					String currentAsmPath = toASMresource(atlFile);
					list.add(currentAsmPath);
				}// end of if (object instanceof AtlRes)
			}
		}

		return list;
	}

	/**
	 * 
	 * @param atlFile
	 * @return
	 */
	private static String toASMresource(String atlFile) {
		String currentAsmPath = atlFile.substring(0, atlFile.length() - 3) + "asm";
		return currentAsmPath;
	}

	/**
	 * collectes the ModelHandler
	 * 
	 * @param call
	 * @return
	 */
	private static Map<String, String> collectModelHandler(Call call) {
		Map<String, String> path = new HashMap<String, String>();
		List<Model> in = call.getIn();
		List<Model> out = call.getOut();

		for (int i = 0; i < in.size(); i++) {
			EObject eo = (EObject) in.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				path.put(m.getName(), "");
				path.put(m.getMm().getName(), m.getMm().getHandler().getLiteral());
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to in
		for (int i = 0; i < out.size(); i++) {
			EObject eo = (EObject) out.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				path.put(m.getName(), "");
				path.put(m.getMm().getName(), m.getMm().getHandler().getLiteral());
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to out
		return path;
	}

	private Map<String, String> collectModelTypes(Call call) {
		Map<String, String> path = new HashMap<String, String>();
		List<Model> in = call.getIn();
		List<Model> out = call.getOut();

		for (int i = 0; i < in.size(); i++) {
			EObject eo = (EObject) in.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				path.put(m.getName(), "MODELINPUT");
				path.put(m.getMm().getName(), "METAMODELINPUT");
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to in
		for (int i = 0; i < out.size(); i++) {
			EObject eo = (EObject) out.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				path.put(m.getName(), "MODELOUTPUT");
				path.put(m.getMm().getName(), "METAMODELOUTPUT");
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to out
		return path;
	}

	private Map<String, String> collectPath(Call call) {
		Map<String, String> path = new HashMap<String, String>();

		List<Model> in = call.getIn();
		collectPath(path, in);
		List<Model> out = call.getOut();
		collectPath(path, out);
		return path;
	}

	/**
	 * @param path
	 * @param in
	 */
	private void collectPath(Map<String, String> path, List<? extends Artifact> in) {
		for (int i = 0; i < in.size(); i++) {
			EObject eo = (EObject) in.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				path.put(m.getName(), m.getUri());
				path.put(m.getMm().getName(), m.getMm().getUri());
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to in
	}

	/**
	 * @param map
	 * @param in
	 * @param info
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private Map<String, String> collectModel2MM(EList<Model> in) {
		Map<String, String> models = new HashMap();
		for (int i = 0; i < in.size(); i++) {
			EObject eo = (EObject) in.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				models.put(m.getName(), m.getMm().getName());
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to in
		return models;
	}

	/**
	 * added the libs to the map
	 * 
	 * @param call
	 * @param libs
	 */
	private Map<String, String> collectLibs(Call call) {
		EList<AtlLibary> libarys = call.getLibarys();
		libarys.addAll(call.getEnviorment().getDefaultLib());
		Map<String, String> libs = new HashMap<String, String>();

		for (int i = 0; i < libarys.size(); i++) {
			AtlLibary lib = (AtlLibary) libarys.get(i);
			String asmresource = toASMresource(lib.getUri());
			// lib.getUri()
			libs.put(lib.getName(), asmresource);
			if (true)
				continue;
		}// end of for i to in

		return libs;
	}

	/**
	 * 
	 * @param callableToStart
	 */
	public void startSingleTransformation(final Callable callableToStart) {

		Job job = new SingleTransformationJob("Start transformation :", callableToStart);
		job.setUser(true);
		job.schedule();
		try {
			ResourcesPlugin.getWorkspace().getRoot().refreshLocal(2, null);
		} catch (CoreException e) {
			CallchainEditPlugin.log("Error while refreshing workspace", IStatus.ERROR, e);
		}

	}

	public void startSingleTransformation(Callable callableToStart, IProgressMonitor monitor) {
		SingleTransformationJob job = new SingleTransformationJob("Start transformation :", callableToStart);
		try {
			job.runInWorkspace(monitor);
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void executeTransformations(Callable callableToStart, List<String> skipList, IProgressMonitor monitor) {
		executorBuilder.getCalls().clear();
		TransformationChainJob job = new TransformationChainJob("Start transformation :", callableToStart, skipList);
		job.runInWorkspace(monitor);

	}

}
