/**
 * 
 */
package de.urszeidler.eclipse.callchain.executors;

import org.eclipse.emf.common.util.EList;

import de.urszeidler.eclipse.callchain.Callable;
import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.ExternalCallable;
import de.urszeidler.eclipse.callchain.Property;
import de.urszeidler.eclipse.callchain.PropertyMapper;

/**
 * @author urs
 *
 */
public class StartExternalCallableStep implements ExecutorStep {

	private ExternalCallable ec;

	public StartExternalCallableStep(ExternalCallable object) {
		this.ec = object;
	}

	/* (non-Javadoc)
	 * @see de.urszeidler.eclipse.callchain.executors.ExecutorStep#generate(java.lang.Object)
	 */
	@Override
	public void generate(Object monitor) {
		Callable delegatedCallable = ec.getDelegatedCallable();
		if (delegatedCallable == null)
			return;

		Calls currentCalls = (Calls) ec.eContainer();
		Calls externalCalls = (Calls) delegatedCallable.eContainer();
		if (currentCalls != externalCalls) 
			externalCalls.setSubtitutionMode(true);

		EList<PropertyMapper> mappers = ec.getMappers();
		for (PropertyMapper propertyMapper : mappers) {
			Property input = propertyMapper.getInput();
			Property output = propertyMapper.getOutput();
			if (input != null & output != null) {
				output.setValue(input.getValue());
			}
		}

	}

	/* (non-Javadoc)
	 * @see de.urszeidler.eclipse.callchain.executors.ExecutorStep#getTaskName()
	 */
	@Override
	public String getTaskName() {
		return "Starting "+ec.getName();
	}

	/* (non-Javadoc)
	 * @see de.urszeidler.eclipse.callchain.executors.ExecutorStep#getWrapedObject()
	 */
	@Override
	public Object getWrapedObject() {
		return null;
	}

	/* (non-Javadoc)
	 * @see de.urszeidler.eclipse.callchain.executors.ExecutorStep#isRefreshWorkspace()
	 */
	@Override
	public boolean isRefreshWorkspace() {
		return false;
	}

}
