/*
 * (c) urs zeidler
 */
package de.urszeidler.eclipse.callchain.lauching.ui;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.databinding.observable.value.IValueChangeListener;
import org.eclipse.core.databinding.observable.value.ValueChangeEvent;
import org.eclipse.core.databinding.observable.value.WritableValue;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTabGroup;
import org.eclipse.debug.ui.CommonTab;
import org.eclipse.debug.ui.ILaunchConfigurationDialog;
import org.eclipse.debug.ui.ILaunchConfigurationTab;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;

import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.lauching.core.CallchainConfiguration;
import de.urszeidler.eclipse.callchain.lauching.core.CallchainLaunchConfigurationDelegate;
import de.urszeidler.eclipse.callchain.provider.CallchainItemProviderAdapterFactory;

/**
 * <p>
 * CallchainLaunchConfigurationTabGroup
 * </p>
 * Created : 03.06.2008
 * 
 * @author urs
 * @version $Revision$, $Id$
 */
public class CallchainLaunchConfigurationTabGroup extends AbstractLaunchConfigurationTabGroup implements
		IValueChangeListener {

	private IObservableValue resourceUri;
	private Calls calls;
	private AdapterFactoryLabelProvider itemProvider;
	private ComposedAdapterFactory adapterFactory;
	private DataBindingContext dbc;
	private CallchainTab callchainTab;
	private CallchainPopertiesTab callchainPopertiesTab;

	/**
     * 
     */
	public CallchainLaunchConfigurationTabGroup() {		
		List<AdapterFactory> factories = new ArrayList<AdapterFactory>();
		factories.add(new CallchainItemProviderAdapterFactory());
		factories.add(new ResourceItemProviderAdapterFactory());
		factories.add(new ReflectiveItemProviderAdapterFactory());

		adapterFactory = new ComposedAdapterFactory(factories);

		itemProvider = new AdapterFactoryLabelProvider(getAdapterFactory());
		dbc = new DataBindingContext();

		resourceUri = new WritableValue(null, String.class);

		resourceUri.addValueChangeListener(this);
		// property = EMFProperties.list(CallchainPackage.Literals.CALLS__CALL);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.debug.ui.ILaunchConfigurationTabGroup#createTabs(org.eclipse
	 * .debug.ui.ILaunchConfigurationDialog, java.lang.String)
	 */
	public void createTabs(ILaunchConfigurationDialog dialog, String mode) {

		callchainTab = new CallchainTab(this);
		callchainPopertiesTab = new CallchainPopertiesTab(this);
		ILaunchConfigurationTab[] tabs = new ILaunchConfigurationTab[] { callchainTab,
				callchainPopertiesTab, new CommonTab() };
		setTabs(tabs);
	}

	@Override
	public void initializeFrom(ILaunchConfiguration configuration) {

		try {
			if (configuration.getAttributes().containsKey(CallchainConfiguration.CALLCHAINURI)) {
				String resourceUri_local = (String) configuration.getAttributes().get(
						CallchainConfiguration.CALLCHAINURI);
				// calls =
				// CallchainLaunchConfigurationDelegate.loadCallchain(resourceUri_local);
				// reloadCalls(resourceUri_local);
				resourceUri.setValue(resourceUri_local);
			}

		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		super.initializeFrom(configuration);

	}

	
	
	public String getResourceUri_asString() {
		if(resourceUri==null || resourceUri.getValue()==null)
			return null;
		
		return resourceUri.getValue().toString();
	}

	public Calls getCalls() {
		return calls;
	}

	private void reloadCalls(String text) {
		calls = CallchainLaunchConfigurationDelegate.loadCallchain(getResourceUri_asString());
		// IObservableList mObs = list.observe(calls);
		callchainTab.fillCallableCombobox(calls);
		callchainPopertiesTab.setCalls(calls);
		
	}

	public AdapterFactoryLabelProvider getItemProvider() {
		return itemProvider;
	}

	public ComposedAdapterFactory getAdapterFactory() {
		return adapterFactory;
	}

	public IObservableValue getResourceUri() {
		return resourceUri;
	}

	public DataBindingContext getDbc() {
		return dbc;
	}

	@Override
	public void handleValueChange(ValueChangeEvent event) {
//		IObservableValue observableValue = event.getObservableValue();
		reloadCalls(getResourceUri_asString());

	}
}
