/**
 * 
 */
package de.urszeidler.eclipse.callchain.lauching.ui;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.debug.ui.ILaunchConfigurationTab;
import org.eclipse.emf.common.util.EList;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.Property;
import de.urszeidler.eclipse.callchain.lauching.core.CallchainConfiguration;
import de.urszeidler.eclipse.callchain.lauching.core.CallchainLaunchConfigurationDelegate;
import de.urszeidler.eclipse.callchain.lauching.core.PropertyHolder;
import de.urszeidler.eclipse.swt.SWTWidgetFactory;

/**
 * @author urs
 * 
 */
public class CallchainPopertiesTab extends AbstractLaunchConfigurationTab implements ILaunchConfigurationTab {

	private class PropertyHolderEditingSupport extends EditingSupport {
		private CellEditor editor;
		private int column;
		private ArrayList<String> dialogTypes;

		public PropertyHolderEditingSupport(ColumnViewer viewer, int column) {
			super(viewer);
			this.column = column;

			dialogTypes = new ArrayList<String>();
			dialogTypes.add(CallchainConfiguration.NONE);
			dialogTypes.add(CallchainConfiguration.STRING);
			dialogTypes.add(CallchainConfiguration.RESOURCE);
			dialogTypes.add(CallchainConfiguration.CONTAINER);
			dialogTypes.add(CallchainConfiguration.URI);

			switch (column) {
			case 2:
				editor = new TextCellEditor(((TableViewer) viewer).getTable());
				break;
			case 3:
				editor = new ComboBoxCellEditor(((TableViewer) viewer).getTable(), dialogTypes.toArray(new String[] {}));
				break;

			default:
				break;
			}
		}

		@Override
		protected boolean canEdit(Object element) {
			return true;
		}

		@Override
		protected CellEditor getCellEditor(Object element) {
			return editor;
		}

		@Override
		protected Object getValue(Object element) {
			PropertyHolder ph = (PropertyHolder) element;
			switch (column) {
			case 2:
				return ph.getValue();
			case 3:
				if (ph.getDialogType() == null)
					return 0;
				return dialogTypes.indexOf(ph.getDialogType());
			default:
				break;
			}
			return null;
		}

		@Override
		protected void setValue(Object element, Object value) {
			PropertyHolder ph = (PropertyHolder) element;
			switch (column) {
			case 2:
				ph.setValue((String) value);
				updateLaunchConfigurationDialog();
				break;
			case 3:
				Integer val = (Integer) value;
				ph.setDialogType(dialogTypes.get(val));
				updateLaunchConfigurationDialog();
				break;
			default:
				break;
			}
			getViewer().update(element, null);

		}
	}

	private CallchainLaunchConfigurationTabGroup tabGroup;
	private TableViewer viewer;
	private Map<String, PropertyHolder> map = new HashMap<String, PropertyHolder>();


	public CallchainPopertiesTab(CallchainLaunchConfigurationTabGroup callchainLaunchConfigurationTabGroup) {
		super();
		tabGroup = callchainLaunchConfigurationTabGroup;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.debug.ui.ILaunchConfigurationTab#createControl(org.eclipse
	 * .swt.widgets.Composite)
	 */
	@Override
	public void createControl(Composite parent) {
		Composite comp = new Composite(parent, SWT.NONE);
		setControl(comp);
		comp.setLayout(new GridLayout(1, true));
		comp.setLayoutData(new GridData(1, 1, true, true));
		comp.setFont(parent.getFont());

		createPropetyGrid(comp);

	}

	private void createPropetyGrid(Composite parent) {
		Group group = SWTWidgetFactory.createGroup(parent, "defined properties", 5, 8, GridData.FILL_BOTH);
		Composite comp = SWTWidgetFactory.createComposite(group, 5, 8, GridData.FILL_BOTH);
		GridLayout ld = (GridLayout) comp.getLayout();
		ld.marginWidth = 1;
		ld.marginHeight = 1;

		GridData gd = null;

		// CallchainAdapterFactory callchainAdapterFactory = new
		// CallchainAdapterFactory();

		// skippList = new org.eclipse.swt.widgets.List(parent, SWT.MULTI);
		viewer = new TableViewer(comp, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);

		// viewer = CheckboxTableViewer.newCheckList(comp, SWT.MULTI);

		gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = 5;
		gd.verticalSpan = 7;
		gd.verticalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;

		viewer.getTable().setLayoutData(gd);
		viewer.getTable().setHeaderVisible(true);

		TableViewerColumn name_column = new TableViewerColumn(viewer, SWT.LEFT);
		name_column.getColumn().setText("name ");
		name_column.getColumn().setWidth(250);
		TableViewerColumn dValue_column = new TableViewerColumn(viewer, SWT.LEFT);
		dValue_column.getColumn().setText("default value ");
		dValue_column.getColumn().setWidth(250);
		TableViewerColumn value_column = new TableViewerColumn(viewer, SWT.LEFT);
		value_column.getColumn().setText("value ");
		value_column.getColumn().setWidth(250);
		value_column.setEditingSupport(new PropertyHolderEditingSupport(viewer, 2));

		TableViewerColumn question_column = new TableViewerColumn(viewer, SWT.CENTER);
		question_column.getColumn().setText("dialog");
		question_column.getColumn().setWidth(100);
		question_column.setEditingSupport(new PropertyHolderEditingSupport(viewer, 3));

		viewer.setContentProvider(new IStructuredContentProvider() {

			public void dispose() {
			}

			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}

			public Object[] getElements(Object inputElement) {
				if (inputElement instanceof List<?>) {
					List<?> list = (List<?>) inputElement;
					return list.toArray();
				}// end of if (inputElement instanceof List)
				if (inputElement instanceof Map<?, ?>) {
					Map<?, ?> map = (Map<?, ?>) inputElement;
					Collection<?> collection = map.values();
					ArrayList<PropertyHolder> list = new ArrayList<PropertyHolder>();
					for (Object object : collection) {
						if (object instanceof PropertyHolder) {
							PropertyHolder ph = (PropertyHolder) object;
							if (ph.getProperty() != null)
								list.add(ph);
						}
					}
					return list.toArray();
				}// end of if (inputElement instanceof List)
				return null;
			}

		});

		viewer.setLabelProvider(new ITableLabelProvider() {

			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				return null;
			}

			@Override
			public String getColumnText(Object element, int columnIndex) {
				if (element instanceof PropertyHolder) {
					PropertyHolder ph = (PropertyHolder) element;

					switch (columnIndex) {
					case 0:
						return ph.getProperty().getName();
					case 1:
						return ph.getProperty().getDefaultValue();
					case 2:
						return ph.getValue();
					case 3:
						if(ph.getDialogType()==null)
							return CallchainConfiguration.NONE;
						
						return ph.getDialogType();

					default:
						break;
					}

				}
				return tabGroup.getItemProvider().getColumnText(element, columnIndex);
			}

			@Override
			public void addListener(ILabelProviderListener listener) {
			}

			@Override
			public void dispose() {
			}

			@Override
			public boolean isLabelProperty(Object element, String property) {
				if (element instanceof PropertyHolder) {
					PropertyHolder ph = (PropertyHolder) element;
					return tabGroup.getItemProvider().isLabelProperty(ph.getProperty(), property);
				}
				return tabGroup.getItemProvider().isLabelProperty(element, property);
			}

			@Override
			public void removeListener(ILabelProviderListener listener) {
			}

		});

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
	 */
	@Override
	public String getName() {
		return "set property values";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.debug.ui.ILaunchConfigurationTab#initializeFrom(org.eclipse
	 * .debug.core.ILaunchConfiguration)
	 */
	@Override
	public void initializeFrom(ILaunchConfiguration configuration) {
		try {
			//map.clear();
			if (configuration.getAttributes().containsKey(CallchainConfiguration.PROPERTIESNAMES)) {
				List<?> list = configuration.getAttribute(CallchainConfiguration.PROPERTIESNAMES,
						new ArrayList<String>());
				for (Object object : list) {
					PropertyHolder propertyHolder = map.get(object.toString());
					if (propertyHolder == null)
						propertyHolder = CallchainLaunchConfigurationDelegate.createPropertyHolder(object.toString());

					propertyHolder.setValue(configuration.getAttribute(propertyHolder.getName()
							+ CallchainConfiguration.PROPERTY_VALUE, (String) null));
					propertyHolder.setDialogType(configuration.getAttribute(propertyHolder.getName()
							+ CallchainConfiguration.PROPERTY_EDITOR_TYPE, "none"));
					map.put(propertyHolder.getName(), propertyHolder);
				}

			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		if (viewer != null){
			viewer.setInput(map);

		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse
	 * .debug.core.ILaunchConfigurationWorkingCopy)
	 */
	@Override
	public void performApply(ILaunchConfigurationWorkingCopy configuration) {

		ArrayList<String> list = new ArrayList<String>(map.keySet());
		configuration.setAttribute(CallchainConfiguration.PROPERTIESNAMES, list);

		for (String string : list) {
			PropertyHolder propertyHolder = map.get(string);
			if (propertyHolder.getValue() != null)
				configuration.setAttribute(propertyHolder.getName() + CallchainConfiguration.PROPERTY_VALUE,
						propertyHolder.getValue());
			if (propertyHolder.getDialogType() != null)
				configuration.setAttribute(propertyHolder.getName() + CallchainConfiguration.PROPERTY_EDITOR_TYPE,
						propertyHolder.getDialogType());
		}
	}
	
	/**
	 * extract the properties and update the config-
	 * @param calls
	 */
	public void setCalls(Calls calls) {
		if (calls == null || viewer == null)
			return;

		EList<Property> properties = calls.getProperties();
		updatePropertyMap(properties);
		viewer.setInput(map);
		updateLaunchConfigurationDialog();
	}

	private void updatePropertyMap(EList<Property> properties) {
		map.clear();
		for (Property property : properties) {
			String name = property.getName();
			PropertyHolder pHolder = map.get(name);
			if (pHolder == null) {
				pHolder = CallchainLaunchConfigurationDelegate.createPropertyHolder(property);
				map.put(name, pHolder);
			} else
				pHolder.setProperty(property);
		}
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(org.eclipse.
	 * debug.core.ILaunchConfigurationWorkingCopy)
	 */
	@Override
	public void setDefaults(ILaunchConfigurationWorkingCopy configuration) {
	}

//	/* (non-Javadoc)
//	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getImage()
//	 */
//	public Image getImage() {
//		 CallchainEditPlugin.getPlugin().getImage("icon/Property.gif");
//		return DebugUITools.getImage(IInternalDebugUIConstants.IMG_OBJS_COMMON_TAB);
//	}

}
