/**
 * 
 */
package de.urszeidler.eclipse.callchain.topcasedgenerators;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.topcased.generator.AbstractGenerator;
import org.topcased.generator.util.ConfiguratorObjectManager;
import org.topcased.modeler.diagramconfigurator.DiagramConfiguration;
import org.topcased.modeler.generator.internal.generators.DiagramConfiguratorGenerator;

import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.editor.preferences.UriFieldEditor;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;
import de.urszeidler.eclipse.callchain.generatorservice.executables.AbstractExecuteable;
import de.urszeidler.eclipse.callchain.topcasedgenerators.preferences.PreferenceConstants;

/**
 * @author urs
 *
 */
public class DiagramGenerator extends AbstractExecuteable  implements GeneratorExecutable {

	public static final String TC_DIAGRAM_CONFIGURATOR_0_10 = "http://www.topcased.org/diagramconfigurator/0.10";
	public static final String TC_DIAGRAM_CONFIGURATOR_1_0 = "http://www.topcased.org/diagramconfigurator/1.0";
	public static final String[] TC_DIAGRAM_CONFIGURATORS = Activator.getDefault().getPreferenceStore().getString(PreferenceConstants.DIAGRAM_URIS).split(UriFieldEditor.LIST_SPILTT) ;
	/**
	 * 
	 */
	public DiagramGenerator() {
	}

	/* (non-Javadoc)
	 * @see de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#configure(de.urszeidler.eclipse.callchain.Generic_Generator)
	 */
	@Override
	public void configure(Object generator) {
		super.configure(generator);

	}
	@Override
	protected String validateGG(Generic_Generator generator) {
		Model model =  findModelByMetaModel(generator.getUses(), TC_DIAGRAM_CONFIGURATORS);
		if (model == null)
			return "No used model conforms to "+TC_DIAGRAM_CONFIGURATOR_0_10+"or "+TC_DIAGRAM_CONFIGURATOR_1_0+" Metamodel.";
		
		return "";
	}

	/* (non-Javadoc)
	 * @see de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#execute(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void execute(IProgressMonitor monitor) throws CoreException {
		Model model = findModelByMetaModel(generator.getUses(), TC_DIAGRAM_CONFIGURATORS);
		if (model == null)
			return;

			String uri = model.getUri();
			IWorkspace wks = ResourcesPlugin.getWorkspace();
			IWorkspaceRoot wksroot = wks.getRoot();

			IFile currentFile = wksroot.getFile(new Path(uri));
			try {

				ConfiguratorObjectManager manager = new ConfiguratorObjectManager();
				manager.load(currentFile.getFullPath());
				// retrieve the root model object and check if it has the expected
				// type
				if (manager.getRootModelObject() instanceof DiagramConfiguration) {
					DiagramConfiguration configuration = (DiagramConfiguration) manager
							.getRootModelObject();
					DiagramConfiguratorGenerator generator = new DiagramConfiguratorGenerator(
							configuration);
					IProject generatedProject = generator.generate(monitor);

					if (configuration != null && generatedProject != null)
						if (configuration.isOrganizeImports())
							AbstractGenerator.organizeImports(generatedProject);

				}
			} catch (Exception e) {
				throw new CoreException(new Status(IStatus.ERROR, "de.urszeidler.eclipse.callchain.topcasedgenerators",
						"error while generating diagram pluging", e));
			}
		}





}
