/**
 * 
 */
package de.urszeidler.eclipse.callchain.ui.wizards;

import java.net.MalformedURLException;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.ide.IDE;

import de.urszeidler.eclipse.callchain.provider.CallchainEditPlugin;

/**
 * @author urs
 * 
 */
public class UpdateCallChainWizard extends BasicTemplateWizard {
	private static final String BASE_BUNDEL_ENTRY = "/templates/callchain_update";
	private IStructuredSelection selection;
	private CallchainSimpleTemplatePage diagPage;
	private IFile createdFile;

	public void init(IWorkbench workbench, IStructuredSelection sel) {
		createdFile = null;
		selection = sel;

		setDefaultPageImageDescriptor(WizardsPlugin.imageDescriptorFromPlugin(
				WizardsPlugin.getId(), "NEW_PAGE_WZD"));
		// .getImageDescriptor("NEW_PAGE_WZD"));
		setDialogSettings(WizardsPlugin.getDefault().getDialogSettings());
		setWindowTitle("Create new Callchain diagrams");

	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages() {
		diagPage = new CallchainSimpleTemplatePage(
				"New callchainEd Diagram", selection,"callchain");
		diagPage.setTitle("Callchain Diagrams");
		diagPage.setDescription("Define the model diagram informations.");
		diagPage.setFileExtension("callchain");
		diagPage.setImageDescriptor(WizardsPlugin.imageDescriptorFromPlugin(
                WizardsPlugin.getId(), "icons/callchain64.png"));
		//diagPage.setSourceFileExtension("callchain");
		addPage(diagPage);
	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#canFinish()
	 * @generated
	 */
	public boolean canFinish() {
		return diagPage.isPageComplete();
	}

	protected IContainer getTargetContainer() {
		return diagPage.getSelectedIContainer();
	}

	protected String getModelName() {
		return diagPage.getModelName();
	}
	protected String getBaseTemplatePath() {
		return BASE_BUNDEL_ENTRY;
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		boolean result = true;
		if (diagPage.isPageComplete()) {
			try {
				result = result & createFilesFromTemplate(new NullProgressMonitor());

			} catch (CoreException e) {
				CallchainEditPlugin.log(e);
			} catch (MalformedURLException e) {
				CallchainEditPlugin.log(e);
			}
			// result = result & createDiagramFile();
			if (createdFile != null && result) {
				
				// Open the newly created model
				try {
					IDE.openEditor(WizardsPlugin.getDefault().getWorkbench()
							.getActiveWorkbenchWindow().getActivePage(),
							createdFile, true);
				} catch (PartInitException pie) {
					CallchainEditPlugin.log(pie);
				}				
			}
            if(diagPage.isAddAltNature())
                addATLnature(null, diagPage.getNatureType());
		}
		return result;
	}
	@Override
	protected String getProjectName() {
		return diagPage.getProjectName();
	}

	@Override
	protected String getModelFile() {
		return diagPage.getModelFile();
	}

}
