/**
 * 
 */
package de.urszeidler.eclipse.callchain.ui.wizards.templates;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.eclipse.ui.dialogs.ResourceListSelectionDialog;

import de.urszeidler.eclipse.callchain.ui.wizards.CallchainSimpleTemplatePage;
import de.urszeidler.eclipse.callchain.ui.wizards.WizardsPlugin;

/**
 * @author urs
 * 
 */
public class InialiseEEFTemplatePage extends CallchainSimpleTemplatePage {

	private static final String ATL_FLOW_TRANSFORMATIONS_ECORE_TO_GMF_GMF = "atl flow transformations/eef-process";
	private static final String DEFAULT_GMF_TEMPLATE_PATH = "DEFAULT_GMF_TEMPLATE_PATH";

	public InialiseEEFTemplatePage(String pageName,
			IStructuredSelection selection, String sourceFileExtension) {
		super(pageName, selection, sourceFileExtension);
		setTitle("EEF from emf genmodel");
		setDescription("Creates an atl custom file and coresponding start configurations.");
		setImageDescriptor(WizardsPlugin.imageDescriptorFromPlugin(WizardsPlugin.getId(), "icons/AtlflowDocument_template_newWizard_org.png"));

	}

	public InialiseEEFTemplatePage(String pageName,
			IStructuredSelection selection) {
		super(pageName, selection);
	}

	private Group basicParamterTemplateGroup;
	private Text templatePath;
	private Button templatePathBt;
//	private Text outputPath;
	private Button outputPathBt;
	private Text modelUri;
	private Button modelUriBt;

	public InialiseEEFTemplatePage(String pageName) {
		super(pageName, null);
	}

	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	@Override
	public void createControl(Composite parent) {
		Composite mainComp = new Composite(parent, SWT.NONE);
		mainComp.setLayout(new GridLayout());
		mainComp.setLayoutData(new GridData(GridData.FILL_BOTH));
		mainComp.setFont(parent.getFont());

		createBasicTemplateParameterGroup(mainComp);
		createProjectNatureGoup(mainComp);

		// Initialize widgets value
		loadData();
		hookListener();

		// Show description on opening
		setErrorMessage(null);
		setMessage(null);
		setControl(mainComp);

		setPageComplete(validatePage());

	}

	private void hookListener() {
		outputPathBt.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleContainerChoosen(outputPath, "Choose the output path :");
			}
		});

		templatePathBt.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleContainerChoosen(templatePath, "Choose the template path :");
			}
		});

		modelUriBt.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleUriChoose();
			}
		});
	}

	protected void handleUriChoose() {
		IContainer container =  ResourcesPlugin.getWorkspace().getRoot();
		int typeMask = IResource.FILE;
		ResourceListSelectionDialog resourceListSelectionDialog = new ResourceListSelectionDialog(getShell(), container, typeMask);
		if(resourceListSelectionDialog.open()==Dialog.OK){
			Object[] results = resourceListSelectionDialog.getResult();
			if (results.length == 1 && results[0] instanceof IFile) {
				IFile file = (IFile) results[0];
				modelUri.setText(file.toString());
				setnameFdText(file.getName().substring(0,
						file.getName().length() - file.getFileExtension().length() - 1));

			}
		}
	}

	protected void handleContainerChoosen(Text textField, String message) {
		IResource resource = ResourcesPlugin.getWorkspace().getRoot()
				.findMember(textField.getText());
		IContainer container = ResourcesPlugin.getWorkspace().getRoot();
		if (resource instanceof IContainer) {
			container = (IContainer) resource;

		}// end of if (resource instanceof IContainer)
		
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(
				getShell(), container, true, message);
		if (dialog.open() == Window.OK) {
			Object[] results = dialog.getResult();
			if (results.length == 1 && results[0] instanceof IPath) {
				IPath newPath = (IPath) results[0];
				textField.setText(newPath.toString());
			}
		}
	}

	private void createBasicTemplateParameterGroup(Composite parent) {
		
		basicParamterTemplateGroup = new Group(parent, SWT.NONE);
		basicParamterTemplateGroup.setLayout(new GridLayout(3, false));
		basicParamterTemplateGroup.setLayoutData(new GridData(
				GridData.FILL_HORIZONTAL));
		basicParamterTemplateGroup.setText("Basic Parameters");
		
		Label nameLbl = new Label(basicParamterTemplateGroup, SWT.NONE);
		nameLbl.setText("name :");
		nameFd = new Text(basicParamterTemplateGroup, SWT.BORDER);
		nameFd.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		new Label(basicParamterTemplateGroup, SWT.NONE);
		
		// The new name of the model file to create
		Label uriLbl = new Label(basicParamterTemplateGroup, SWT.NONE);
		uriLbl.setText("Input Model name : ");

		modelUri = new Text(basicParamterTemplateGroup, SWT.BORDER);
		modelUri.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		modelUriBt = new Button(basicParamterTemplateGroup, SWT.PUSH);
		modelUriBt.setText(BROWSE_TEXT);

		Label outputLbl = new Label(basicParamterTemplateGroup, SWT.NONE);
		outputLbl.setText("output path : ");

		outputPath = new Text(basicParamterTemplateGroup, SWT.BORDER);
		outputPath.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		outputPathBt = new Button(basicParamterTemplateGroup, SWT.PUSH);
		outputPathBt.setText(BROWSE_TEXT);

		Label directoryLbl = new Label(basicParamterTemplateGroup, SWT.NONE);
		directoryLbl.setText("Path to templates : ");

		templatePath = new Text(basicParamterTemplateGroup, SWT.BORDER);
		templatePath.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		templatePathBt = new Button(basicParamterTemplateGroup, SWT.PUSH);
		templatePathBt.setText(BROWSE_TEXT);

	}

	protected void loadData() {
		if (selectedResource instanceof IFile) {
			// existingModelBt.setSelection(true);
			// newModelGrp.setEnabled(false);
			IFile file = (IFile) selectedResource;
			selectedFile = file.getFullPath().toString();

			String pathtoTemplate = "";
			templatePath.setText(pathtoTemplate);
			modelUri.setText(selectedFile);
			setnameFdText(file.getName().substring(0,
					file.getName().length() - file.getFileExtension().length() - 1));


		} else if (selectedResource instanceof IContainer) {
			IContainer container = (IContainer) selectedResource;
			templatePath.setText(container.getFullPath().toString());

		}
		IDialogSettings dialogSettings = getDialogSettings().getSection(NewCallchainDiagrams_EEFProcessFromTemplate.INITALISE_EEF_TEMPLATE_SETTINGS);
		if(dialogSettings==null)
			dialogSettings = getDialogSettings().addNewSection(NewCallchainDiagrams_EEFProcessFromTemplate.INITALISE_EEF_TEMPLATE_SETTINGS);
			
		String defaultTemplatePath = dialogSettings.get(DEFAULT_GMF_TEMPLATE_PATH);
		if(defaultTemplatePath==null)
			defaultTemplatePath=ATL_FLOW_TRANSFORMATIONS_ECORE_TO_GMF_GMF;
		
		templatePath.setText(defaultTemplatePath);
	}

	/**
	 * Check if the informations contained in the page are OK
	 * 
	 * @return true is OK
	 */
	protected boolean validatePage() {
		setMessage(null);
		setErrorMessage(null);
		return true;
	}


	
	public Text getModelUri() {
		return modelUri;
	}

	public void setModelUri(Text modelUri) {
		this.modelUri = modelUri;
	}

	public Text getTemplatePath() {
		return templatePath;
	}

	public Text getOutputPath() {
		return outputPath;
	}

}
