/**
 * 
 */
package de.urszeidler.eclipse.callchain.ui.wizards.templates;

import java.net.MalformedURLException;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.gmf.internal.bridge.resolver.StructureResolver;
import org.eclipse.gmf.internal.bridge.wizards.Messages;
import org.eclipse.gmf.internal.bridge.wizards.pages.simple.DefinitionPage;
import org.eclipse.gmf.internal.bridge.wizards.pages.simple.DomainModelSelectionPage;
import org.eclipse.gmf.internal.bridge.wizards.pages.simple.StructureBuilder;
import org.eclipse.gmf.internal.common.ui.ResourceLocationProvider;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.ide.IDE;

import de.urszeidler.eclipse.callchain.provider.CallchainEditPlugin;
import de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard;
import de.urszeidler.eclipse.callchain.ui.wizards.WizardsPlugin;

/**
 * @author urs
 * 
 */
public class NewCallchainDiagrams_EEFProcessFromTemplate extends
BasicTemplateWizard {
	public static final String INITALISE_EEF_TEMPLATE_SETTINGS = "Initalise_EEF_TEMPLATE";

	private static final String BASE_BUNDEL_ENTRY = "/templates/eef-process-template";

	private InialiseEEFTemplatePage inialiseEEFTemplatePage;

	private IDialogSettings dialogSettings;

	private IStructuredSelection selection;
	private IFile createdFile;

	public NewCallchainDiagrams_EEFProcessFromTemplate(
			IStructuredSelection selection) {
		super();
		this.selection = selection;
		IDialogSettings dialogSettings = getDialogSettings().getSection(INITALISE_EEF_TEMPLATE_SETTINGS);
		if(dialogSettings==null)
			dialogSettings = getDialogSettings().addNewSection(INITALISE_EEF_TEMPLATE_SETTINGS);
		
		setDialogSettings(dialogSettings);
		

	
	}

	@Override
	public void init(IWorkbench workbench, IStructuredSelection sel) {

	}

	
	@SuppressWarnings("restriction")
	@Override
	public void addPages() {
		inialiseEEFTemplatePage = new InialiseEEFTemplatePage(
				"select basic parameters", this.selection, "genmodel");
		
		inialiseEEFTemplatePage.setTitle("EEF template process");
		inialiseEEFTemplatePage.setDescription("Creates a start configuration and a custom atl file for the eef template process.");
		addPage(inialiseEEFTemplatePage);
	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#canFinish()
	 */
	public boolean canFinish() {
		return inialiseEEFTemplatePage.isPageComplete();
	}

	@Override
	protected void initVariables(String name, String cname, String pname,
			String modelFile) {
		super.initVariables(name, cname, pname, modelFile);
		
		_variables.put("model_uri", inialiseEEFTemplatePage.getModelUri().getText());
		_variables.put("output_path", inialiseEEFTemplatePage.getOutputPath().getText());
		_variables.put("template_path", inialiseEEFTemplatePage.getTemplatePath().getText());
		
	}
	
	protected IContainer getTargetContainer() {
		return inialiseEEFTemplatePage.getSelectedIContainer();
	}
	
	@Override
	protected String getBaseTemplatePath() {
		return BASE_BUNDEL_ENTRY;
	}
	@Override
	protected String getProjectName() {
		Text outputPath = inialiseEEFTemplatePage.getOutputPath();
		IResource member = ResourcesPlugin.getWorkspace().getRoot().findMember(outputPath.getText());
		IProject project = member.getProject();
		return project.getName();
	}

	@Override
	protected String getModelName() {
		return inialiseEEFTemplatePage.getModelName();
	}

	@Override
	protected String getModelFile() {
		return inialiseEEFTemplatePage.getModelFile();
	}
	
	/**
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	public boolean performFinish() {
		boolean result = true;
		if (canFinish()) {
			//saveDialogSettings();
			try {
				result = result & createFilesFromTemplate(new NullProgressMonitor());

			} catch (CoreException e) {
				CallchainEditPlugin.log(e);
			} catch (MalformedURLException e) {
				CallchainEditPlugin.log(e);
			}
			// result = result & createDiagramFile();
			if (createdFile != null && result) {
				// Open the newly created model
				try {
					IDE.openEditor(WizardsPlugin.getDefault().getWorkbench()
							.getActiveWorkbenchWindow().getActivePage(),
							createdFile, true);
				} catch (PartInitException pie) {
					CallchainEditPlugin.log(pie);
				}
			}
			if (inialiseEEFTemplatePage.isAddAltNature())
				addATLnature(null, inialiseEEFTemplatePage.getNatureType());
		}
		
		
		return result;
	}

	
	
}
