/*
 * (c) urs zeidler
 */
package de.urszeidler.eclipse.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

/**
 * 
 * <p> 
 * SWTWidgetFactory, a simple widget Factory for the default swt widgets 
 * </p> 
 * Created : 01.11.2007 
 * @author urs 
 * @version $Revision$, $Id$
 */
public class SWTWidgetFactory {

	/**
     * Creates a new text widget 
     * @param parent the parent composite to add this text widget to
     * @return the new text widget
     * @since 3.2
     * 
     */
    public static Text createSingleText(Composite parent,Object data) {
        Text t = new Text(parent, SWT.SINGLE | SWT.BORDER);
        t.setFont(parent.getFont());
        t.setData(data);
        return t;
    }

    /**
     * Creates a new text widget 
     * @param parent the parent composite to add this text widget to
     * @param hspan the horizontal span to take up on the parent composite
     * @return the new text widget
     * @since 3.2
     * 
     */
    public static Text createSingleText(Composite parent, int hspan,Object data) {
        Text t = createSingleText(parent, hspan);
        t.setData(data);
        return t;
    }

    
    /**
     * Creates a new text widget 
     * @param parent the parent composite to add this text widget to
     * @param hspan the horizontal span to take up on the parent composite
     * @return the new text widget
     * @since 3.2
     * 
     */
    public static Text createSingleText(Composite parent, int hspan) {
        Text t = new Text(parent, SWT.SINGLE | SWT.BORDER);
        t.setFont(parent.getFont());
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = hspan;
        t.setLayoutData(gd);
        return t;
    }
    
    /**
     * Creates a new text widget 
     * @param parent the parent composite to add this text widget to
     * @param style the style bits for the text widget
     * @param hspan the horizontal span to take up on the parent composite
     * @param fill the fill for the grid layout
     * @return the new text widget
     * @since 3.3
     */
    public static Text createText(Composite parent, int style, int hspan, int fill) {
        Text t = new Text(parent, style);
        t.setFont(parent.getFont());
        GridData gd = new GridData(fill);
        gd.horizontalSpan = hspan;
        t.setLayoutData(gd);
        return t;
    }
    /**
     * Creates a Group widget
     * @param parent the parent composite to add this group to
     * @param text the text for the heading of the group
     * @param columns the number of columns within the group
     * @param hspan the horizontal span the group should take up on the parent
     * @param fill the style for how this composite should fill into its parent
     * @return the new group
     * @since 3.2
     * 
     */
    public static Group createGroup(Composite parent, String text, int columns, int hspan, int fill) {
        Group g = new Group(parent, SWT.NONE);
        g.setLayout(new GridLayout(columns, false));
        g.setText(text);
        g.setFont(parent.getFont());
        GridData gd = new GridData(fill);
        gd.horizontalSpan = hspan;
        g.setLayoutData(gd);
        return g;
    }
    
    /**
     * Creates a composite that uses the parents' font and has a grid layout
     * @param parent the parent to add the composite to
     * @param columns the number of columns the composite should have
     * @param hspan the horizontal span the new composite should take up in the parent
     * @param fill the fill style of the composite {@link GridData}
     * @return a new composite with a grid layout
     * 
     * @since 3.3
     */
    public static Composite createComposite(Composite parent, int columns, int hspan, int fill) {
        Composite g = new Composite(parent, SWT.NONE);
        g.setLayout(new GridLayout(columns, false));
        g.setFont(parent.getFont());
        GridData gd = new GridData(fill);
        gd.horizontalSpan = hspan;
        g.setLayoutData(gd);
        return g;
    }

    /**
     * Creates a new label widget
     * @param parent the parent composite to add this label widget to
     * @param text the text for the label
     * @param hspan the horizontal span to take up in the parent composite
     * @return the new label
     * @since 3.2
     * 
     */
    public static Label createLabel(Composite parent, String text, int hspan) {
        Label l = new Label(parent, SWT.NONE);
        l.setFont(parent.getFont());
        l.setText(text);
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = hspan;
        gd.grabExcessHorizontalSpace = false;
        l.setLayoutData(gd);
        return l;
    }

	public static Text createSingleText(Composite parent, Object data,
			Object layoutData) {
		Text t = createSingleText(parent, data);
		t.setLayoutData(layoutData);
		return t;
		
	}

	public static Text createSingleText(Composite parent, Object data,
			Object layoutData, ModifyListener modifyListener) {
		Text t = createSingleText(parent, data);
		t.addModifyListener(modifyListener);
		return t;
	}

    
}
